
#include "Robert_timer.h"


#define TIMER_LOG "TIMER_LOG"

typedef struct {
    int timer_group;
    int timer_idx;
    int alarm_interval;
    bool auto_reload;
} example_timer_info_t;

/**
 * @brief A sample structure to pass events from the timer ISR to task
 *
 */
typedef struct {
    example_timer_info_t info;
    uint64_t timer_counter_value;
} example_timer_event_t;

static xQueueHandle s_timer_queue;

int testTimerDeal  = 0;
int testTimerhandle  = 0;


static bool IRAM_ATTR timer_group_isr_callback(void *args)
{
    example_timer_info_t *info = (example_timer_info_t *)args;
    if(info->timer_group == TIMER_GROUP_0)
    {
        testTimerDeal++;
        timer_Deal();
    }
    else if(info->timer_group == TIMER_GROUP_1)
    {
        testTimerhandle++;
        IQhandle();
    }

    return true;
}

/**
 * @brief Initialize selected timer of timer group
 *
 * @param group Timer Group number, index from 0
 * @param timer timer ID, index from 0
 * @param auto_reload whether auto-reload on alarm event
 * @param timer_interval_sec interval of alarm
 */
void example_tg_timer_init(int group, int timer, bool auto_reload, int timer_interval_sec)
{
    /* Select and initialize basic parameters of the timer */
    timer_config_t config = {
        .divider = TIMER_DIVIDER,
        .counter_dir = TIMER_COUNT_UP,
        .counter_en = TIMER_PAUSE,
        .alarm_en = TIMER_ALARM_EN,
        .auto_reload = auto_reload,
        .intr_type = TIMER_INTR_LEVEL,
    }; // default clock source is APB
    timer_init(group, timer, &config);
    
    /* Timer's counter will initially start from value below.
       Also, if auto_reload is set, this value will be automatically reload on alarm */
    timer_set_counter_value(group, timer, 0);

    /* Configure the alarm value and the interrupt on alarm. */
    timer_set_alarm_value(group, timer, timer_interval_sec * TIMER_SCALE);
    timer_enable_intr(group, timer);
    
    example_timer_info_t *timer_info = calloc(1, sizeof(example_timer_info_t));
    timer_info->timer_group = group;
    timer_info->timer_idx = timer;
    timer_info->auto_reload = auto_reload;
    timer_info->alarm_interval = timer_interval_sec;
    timer_isr_callback_add(group, timer, timer_group_isr_callback, timer_info, 0);
   
    timer_start(group, timer);
}

void timer_user_Init()
{
    s_timer_queue = xQueueCreate(10, sizeof(example_timer_event_t));
    example_tg_timer_init(TIMER_GROUP_0, TIMER_0, true, 5);
    example_tg_timer_init(TIMER_GROUP_1, TIMER_0, true, 1);
    ESP_LOGE(TIMER_LOG,"timer Init start!!");
}

void timer_Deal()
{
    if(ConnectedFlag == 0 && advertisingFlag == 1)
    {
        advertisingtimeCount++;
        // ESP_LOGE(TIMER_LOG,"time count %d",advertisingtimeCount);
        if(advertisingtimeCount >= advertisingtime*60*100)
        {
        // NRF_LOG_INFO("timeout close nrf!!!");
        // sd_ble_gap_adv_stop(m_advertising.adv_handle);
        // esp_ble_gap_start_advertising();
        // ESP_LOGE(TIMER_LOG,"timeout close nrf!!");
        esp_ble_gap_stop_advertising();
        advertisingFlag = 0;
        }
    }
    else
    {
        advertisingtimeCount = 0;
    }
if(ConnectedFlag == 1)
  {
    updateStatusTime++;
  }
  else
  {
    updateStatusTime = 0;
  }
  Leave_Tx_Time++;
  Near_Tx_Time++;
  TX_time++;                //用于定时500mS，发送数据
  //LED_Sensor_Time++;
  if(Near_Tx_Time >=50)    //用于定时500mS
  {
    Near_Tx_Time = 50;
    //GPIOA->ODR &= 0xF7;      //PA3上电输出低电平,蓝或绿灯亮灭
  }
  if(Leave_Tx_Time >=50)    //用于定时500mS
  {
    Leave_Tx_Time = 50;
    //GPIOD->ODR &= 0xEF;      //PD4上电输出低电平,红灯灭
  }
  if(TX_time >=50)    //用于定时500mS
  {
    TX_time = 50;
  }
  if(Leave_CNT_A >0)
  {
    Leave_Time_A++;
    if(Leave_Time_A >80)    //800mS
    {
      Leave_CNT_A =0;
      Leave_Time_A =0;
      
    }
  }
  if(Leave_CNT_B >0)
  {
    Leave_Time_B++;
    if(Leave_Time_B >200)   //2000mS
    {
      Leave_Time_D++;
      Leave_Time_B =0;
      if(Leave_Time_D >60) //120S
      {
        Leave_Tx_update =0;
        Leave_CNT_B =0;
        Leave_Time_D =0;
        Leave_Time_C = 200;
      }
    }
     
  }
  if(Leave_Tx_update)
  {
    Leave_Time_C++;
    if(Leave_Time_C >=80)  //800mS
    {
      Leave_CNT_B = 0;
      Leave_Time_B = 0;
      Leave_Time_D =0;
      if(Leave_Time_C >=90)
      {
        Leave_Time_C = 90;   //只是延时0.9秒
      }
    }
    
  }
  if(Near_CNT >0)
  {
    Near_Time++;
    if(Near_Time >80)    //800mS
    {
      Near_CNT =0;
      Near_Time =0;
      
    }
  }
  if(Output_Sign)
  {
    Output_Time ++;
    if(Output_Time >= Keeptime*100)   //
    {
      Output_Sign =0;
      Output_Time =0;
      //GPIOC->ODR &= 0xBF;                    //PC6输出低电平,光耦继电器常开。
      OUTPUT_L();//2019.11.13 OUTPUT NC
    }
  }
  if(Near_L_CNT > 0)
  {
     Near_L_Time++;
     if(Near_L_Time > 200)   //2000mS
     {
         Near_L_CNT = 0;
         Near_L_Time = 0;      
     }
  }
  if(Near_CNT)
  //if((Near_CNT)||(Leave_CNT_A))
  {
    LED_Sensor_Time++;
    if(LED_Sensor_Time == 1)
    {
      //GPIOC->ODR |= 0x08;      //PC3上电输出高电平
      //GPIOC->ODR &= 0xF7;      //PC3上电输出低电平
      LED_Sensor_L();
    }
    if(LED_Sensor_Time ==(LD2_Time/2))
    {
       //LED_Sensor_Time = 0;
       //GPIOC->ODR &= 0xF7;      //PC3上电输出低电平
       //GPIOC->ODR |= 0x08;      //PC3上电输出高电平
       LED_Sensor_H();
    }
      if(LED_Sensor_Time >= LD2_Time)
      {
        LED_Sensor_Time = 0;
        //GPIOC->ODR |= 0x08;      //PC3上电输出高电平
        //GPIOC->ODR &= 0xF7;      //PC3上电输出低电平
        LED_Sensor_L();
      }
  }
  else
  {
    //GPIOC->ODR &= 0xF7;      //PC3上电输出低电平
    //GPIOC->ODR |= 0x08;      //PC3上电输出高电平
    // LED_Sensor_H();
    LED_Sensor_Time = 0; 
  }
 /*  
  if(LED_Sensor_Time == 1)
  {
    GPIOC->ODR |= 0x08;      //PC3上电输出高电平
  }
  if(LED_Sensor_Time >= 10)
  {
    GPIOC->ODR &= 0xF7;      //PC3上电输出低电平
    
    if(LED_Sensor_Time >= 200)
    {
      LED_Sensor_Time = 0;
      GPIOC->ODR |= 0x08;      //PC3上电输出高电平
    }
  }
  */
  //if(Near_Tx_update)
  //{
   // Near_Time++;
   // if(Near_Time >= 2)  //20mS
    //{
     // Near_Time = 2;
    //}
  //}}
}